# -*- coding: mbcs -*-
# Do not delete the following import lines
from abaqus import *
from abaqusConstants import *
import __main__
import numpy as np

def populate_blocks():
    """
    Instantiate the parts of the geometry described in blocklist_filename.
    In this version, it will only generate the geometry, while element type, boundary conditions, meshing, contact interactions, etc., must be specified by the user in Abaqus.
    """
    import section
    import regionToolset
    import displayGroupMdbToolset as dgm
    import part
    import material
    import assembly
    import step
    import interaction
    import load
    import mesh
    import optimization
    import job
    import sketch
    import visualization
    import xyPlot
    import displayGroupOdbToolset as dgo
    import connectorBehavior

    blocklist_filename = 'guimaraes_blocklist_test' # The input file name
    model_name = 'Model-1'

    block_data = np.load(blocklist_filename + ".npz")
    
    scale_factor = 10. # From 'block units' to meter
    block_sizes = block_data["block_sizes"].copy()/scale_factor
    blocks_ids = block_data["blocks_ids"].copy()
    blocks_translate = block_data["blocks_translate"].copy()/scale_factor
    lintel_sizes = block_data["lintel_sizes"].copy()/scale_factor
    lintel_ids = block_data["lintel_ids"].copy()
    target_space_range = block_data["target_space_range"].copy()/scale_factor

    vol_min_x = target_space_range[0,0]
    vol_max_x = target_space_range[0,1]
    vol_min_y = target_space_range[1,0]
    vol_min_z = np.amin(blocks_translate[:, 2]) # target space range for z is not right because in Abaqus we extrude only in one direction.

    del block_data

    blocks_translate = np.array(blocks_translate)
    blocks_ids = np.array(blocks_ids)

    # Material properties
    elastic_modulus = 4.8*10**9
    poisson_ratio = 0.17
    density = 2700

    # Contact properties
    tangential_mu_x = 0.58
    absolute_distance_x = 0.0005
    contact_max_damage_x = (3*100000, 4.5*100000, 4.5*100000)
    total_plastic_disp_x = 0.001
    visc_coef_x = 0.005
    contact_stiffness_x = 1.*10**12
    cohesion_x = (1.*10000000000, 5.*1000000000, 5.*1000000000)

    tangential_mu_y = 0.58
    absolute_distance_y = 0.0005
    contact_max_damage_y = (3*100000, 4.5*100000, 4.5*100000)
    total_plastic_disp_y = 0.001
    visc_coef_y = 0.005
    contact_stiffness_y = 1.*10**12
    cohesion_y = (1.*10000000000, 5.*1000000000, 5.*1000000000)

    tangential_mu_z = 0.58
    absolute_distance_z = 0.0005
    contact_max_damage_z = (3*100000, 4.5*100000, 4.5*100000)
    total_plastic_disp_z = 0.001
    visc_coef_z = 0.005
    contact_stiffness_z = 1.*10**12
    cohesion_z = (1.*10000000000, 5.*1000000000, 5.*1000000000)

    # Gravity step
    grav_initial_inc = 1.
    grav_max_num_inc = 100000
    grav_min_inc = 1.*10**(-9)

    # Push step
    push_time_period = 100.
    push_max_num_inc = 100000
    push_initial_inc = 0.1
    push_min_inc = 1.*10**(-12)
    push_magnitude = 15000.0

    ## PARTS CREATION
    # Create block parts
    for i, block_size in enumerate(block_sizes):
        part_name = 'block_' + str(i)
        material_name = 'material_' + str(i)
        section_name = 'section_' + str(i)
        part_width = block_size[0]
        part_height = block_size[1]
        part_depth = block_size[2]

        # Create part
        s1 = mdb.models[model_name].ConstrainedSketch(name='__profile__', 
            sheetSize=10.0/scale_factor)
        g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
        s1.setPrimaryObject(option=STANDALONE)
        s1.rectangle(point1=(-part_width/2., part_height/2.), point2=(part_width/2., -part_height/2.))

        p = mdb.models[model_name].Part(name=part_name, dimensionality=THREE_D, 
            type=DEFORMABLE_BODY)
        p.BaseSolidExtrude(sketch=s1, depth=part_depth)
        s1.unsetPrimaryObject()

        # Create material
        mdb.models[model_name].Material(name=material_name)
        mdb.models[model_name].materials[material_name].Elastic(table=((elastic_modulus, poisson_ratio), ))
        mdb.models[model_name].materials[material_name].Density(table=((density, ), ))

        # Create section
        mdb.models[model_name].HomogeneousSolidSection(name=section_name, 
        material=material_name, thickness=None)

        # Assign section
        c = p.cells
        region = regionToolset.Region(cells=c)
        p.SectionAssignment(region=region, sectionName=section_name, offset=0.0, offsetType=MIDDLE_SURFACE, offsetField='', thicknessAssignment=FROM_SECTION)

        del mdb.models[model_name].sketches['__profile__']

    # Create lintel parts
    for i, lintel_size in enumerate(lintel_sizes):
        part_name = 'lintel_' + str(i)
        material_name = 'lintel_material_' + str(i)
        section_name = 'lintel_section_' + str(i)
        part_width = lintel_size[0]
        part_height = lintel_size[1]
        part_depth = lintel_size[2]

        # Create part
        s1 = mdb.models[model_name].ConstrainedSketch(name='__profile__', 
            sheetSize=10.0/scale_factor)
        g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
        s1.setPrimaryObject(option=STANDALONE)
        s1.rectangle(point1=(-part_width/2., part_height/2.), point2=(part_width/2., -part_height/2.))

        p = mdb.models[model_name].Part(name=part_name, dimensionality=THREE_D, 
            type=DEFORMABLE_BODY)
        p.BaseSolidExtrude(sketch=s1, depth=part_depth)
        s1.unsetPrimaryObject()

        # Create material
        mdb.models[model_name].Material(name=material_name)
        mdb.models[model_name].materials[material_name].Elastic(table=((elastic_modulus, poisson_ratio), ))
        mdb.models[model_name].materials[material_name].Density(table=((density, ), ))

        # Create section
        mdb.models[model_name].HomogeneousSolidSection(name=section_name, 
        material=material_name, thickness=None)

        # Assign section
        c = p.cells
        region = regionToolset.Region(cells=c)
        p.SectionAssignment(region=region, sectionName=section_name, offset=0.0, offsetType=MIDDLE_SURFACE, offsetField='', thicknessAssignment=FROM_SECTION)

        del mdb.models[model_name].sketches['__profile__']
    
    a = mdb.models[model_name].rootAssembly

    ## PARTS INSTANTIATION AND TRANSLATION FOR ASSEMBLY
    # Instantiate block parts
    part_instance_count = 0
    base_instance_names = []
    base_instance_pos = []
    right_lateral_instance_names = []
    right_lateral_instance_pos = []
    right_lateral_instance_sizes = []
    left_lateral_instance_names = []
    left_lateral_instance_pos = []
    left_lateral_instance_sizes = []
    selected_cells = None
    all_instances = ()
    all_instance_names = []

    pos_vertices = []
    neg_vertices = []
    instance_sizes = []
    for block_id, block_translate in zip(blocks_ids, blocks_translate):
        part_id = np.round(block_id).astype(np.int32)
        part_name = 'block_' + str(part_id)

        try:
            # Check if instance is at the base or laterals
            block_size = block_sizes[part_id, :]
            part_width = block_size[0]
            part_height = block_size[1]
            part_depth = block_size[2]

            part_min_y = block_translate[1] - part_height/2.
            part_min_z = block_translate[2]
            part_min_x = block_translate[0] - part_width/2.
            part_max_x = block_translate[0] + part_width/2.
            
            part_instance_count = part_instance_count + 1
            part_instance_name = part_name + '_' + str(part_instance_count)
            
            if (vol_min_y==part_min_y):
                base_instance_names.append(part_instance_name)
                base_instance_pos.append(block_translate)

            if (vol_min_z==part_min_z) and (vol_max_x==part_max_x):
                right_lateral_instance_names.append(part_instance_name)
                right_lateral_instance_pos.append(block_translate)                
                right_lateral_instance_sizes.append(block_size)

            if (vol_min_z==part_min_z) and (vol_min_x==part_min_x):
                left_lateral_instance_names.append(part_instance_name)
                left_lateral_instance_pos.append(block_translate)                
                left_lateral_instance_sizes.append(block_size)

            p = mdb.models[model_name].parts[part_name]
            a.Instance(name=part_instance_name, part=p, dependent=OFF)
            a.translate(instanceList=(part_instance_name, ), vector=(block_translate[0], block_translate[1], block_translate[2]))

            part_instance = a.instances[part_instance_name]
            all_instances = all_instances + (part_instance,)
            all_instance_names.append(part_instance_name)
            instance_cells = part_instance.cells
            look_pos = (block_translate[0], block_translate[1], block_translate[2]-vol_min_z/2.)
            select_cell = instance_cells.findAt((look_pos, ))
            
            if selected_cells is None:
                selected_cells = select_cell
            else:
                selected_cells = selected_cells + select_cell

            pos_vertice = [block_translate[0] + part_width/2., block_translate[1] + part_height/2., block_translate[2] + part_depth]
            neg_vertice = [block_translate[0] - part_width/2., block_translate[1] - part_height/2., block_translate[2]]
            instance_size = [part_width, part_height, part_depth]

            pos_vertices.append(pos_vertice)
            neg_vertices.append(neg_vertice)
            instance_sizes.append(instance_size)
        except:
            print("Failed to add part ", part_name)

    # Instantiate lintel parts
    for i, lintel_id in enumerate(lintel_ids):
        part_id = np.round(i).astype(np.int32)
        part_name = 'lintel_' + str(part_id)
        part_instance_count = part_instance_count + 1
        part_instance_name = part_name + '_' + str(part_instance_count)

        block_size = lintel_sizes[part_id, :]
        part_width = block_size[0]
        part_height = block_size[1]
        part_depth = block_size[2]


        block_translate = blocks_translate[np.where(blocks_ids==lintel_id)[0][0], :]
        try:
            p = mdb.models[model_name].parts[part_name]
            a.Instance(name=part_instance_name, part=p, dependent=OFF)
            a.translate(instanceList=(part_instance_name, ), vector=(block_translate[0], block_translate[1], block_translate[2]))

            part_instance = a.instances[part_instance_name]
            all_instances = all_instances + (part_instance,)
            all_instance_names.append(part_instance_name)
            instance_cells = part_instance.cells
            look_pos = (block_translate[0], block_translate[1], block_translate[2]-vol_min_z/2.)
            select_cell = instance_cells.findAt((look_pos, ))
            selected_cells = selected_cells + select_cell
            
            pos_vertice = [block_translate[0] + part_width/2., block_translate[1] + part_height/2., block_translate[2] + part_depth]
            neg_vertice = [block_translate[0] - part_width/2., block_translate[1] - part_height/2., block_translate[2]]
            instance_size = [part_width, part_height, part_depth]

            pos_vertices.append(pos_vertice)
            neg_vertices.append(neg_vertice)
            instance_sizes.append(instance_size)
        except:
            print("Failed to add part ", part_name)

